import { Injectable } from '@angular/core';
import { Helpers } from './helpers';
import { Observer } from './observer';
import { Mirror } from './mirror';
import { ObserverInfo } from './observerinfo';
/**
 * FastJsonPatchService class.
 */
var FastJsonPatchService = /** @class */ (function () {
    function FastJsonPatchService() {
        this.beforeObjects = [];
    }
    /**
     * Generate an array of patches from an observer
     */
    FastJsonPatchService.prototype.generate = function (observer) {
        var mirror = this.getMirror(observer.object);
        if (mirror) {
            Helpers.generate(mirror.originalObject, observer.object, observer.patches, '');
            if (observer.patches.length) {
                Helpers.applyPatch(mirror.originalObject, observer.patches);
            }
            var temp = observer.patches;
            if (temp.length > 0) {
                observer.patches = [];
                if (observer.callback) {
                    observer.callback(temp);
                }
            }
            return temp;
        }
        else {
            throw new Error('Mirror not found');
        }
    };
    FastJsonPatchService.prototype.observe = function (obj, callback) {
        var _this = this;
        var observer;
        var root = obj;
        var mirror = this.getMirror(obj);
        if (!mirror) {
            mirror = new Mirror(obj);
            this.beforeObjects.push(mirror);
        }
        else {
            observer = this.getObserverFromMirror(mirror, callback);
        }
        if (observer) {
            return observer;
        }
        observer = new Observer();
        mirror.originalObject = Helpers.deepClone(obj);
        mirror.obj = obj;
        var dirtyCheck = function () {
            _this.generate(observer);
        };
        var fastCheck = function () {
            clearTimeout(observer.nextHandle);
            observer.nextHandle = setTimeout(dirtyCheck);
        };
        if (callback) {
            observer.callback = callback;
            observer.nextHandle = null;
            if (typeof window !== 'undefined') {
                if (window.addEventListener) {
                    window.addEventListener('mouseup', fastCheck);
                    window.addEventListener('keyup', fastCheck);
                    window.addEventListener('mousedown', fastCheck);
                    window.addEventListener('keydown', fastCheck);
                    window.addEventListener('change', fastCheck);
                }
                else {
                    var documentElement = document.documentElement;
                    documentElement.attachEvent('onmouseup', fastCheck);
                    documentElement.attachEvent('onkeyup', fastCheck);
                    documentElement.attachEvent('onmousedown', fastCheck);
                    documentElement.attachEvent('onkeydown', fastCheck);
                    documentElement.attachEvent('onchange', fastCheck);
                }
            }
        }
        observer.patches = [];
        observer.object = obj;
        observer.unobserve = function () {
            var unobserveMirror = _this.getMirror(observer.object);
            _this.generate(observer);
            clearTimeout(observer.nextHandle);
            _this.removeObserverFromMirror(unobserveMirror, observer);
            if (unobserveMirror.observers.length === 0) {
                // there are no observables anymore so discard the mirror
                var mirrorIndex = _this.beforeObjects.indexOf(unobserveMirror);
                _this.beforeObjects.splice(mirrorIndex, 1);
                unobserveMirror = undefined;
            }
            if (typeof window !== 'undefined') {
                if (window.removeEventListener) {
                    window.removeEventListener('mouseup', fastCheck);
                    window.removeEventListener('keyup', fastCheck);
                    window.removeEventListener('mousedown', fastCheck);
                    window.removeEventListener('keydown', fastCheck);
                }
                else {
                    var documentElement = document.documentElement;
                    documentElement.detachEvent('onmouseup', fastCheck);
                    documentElement.detachEvent('onkeyup', fastCheck);
                    documentElement.detachEvent('onmousedown', fastCheck);
                    documentElement.detachEvent('onkeydown', fastCheck);
                }
            }
        };
        mirror.observers.push(new ObserverInfo(callback, observer));
        return observer;
    };
    FastJsonPatchService.prototype.compare = function (source, target) {
        var patches = [];
        Helpers.generate(source, target, patches, '');
        return patches;
    };
    FastJsonPatchService.prototype.unobserve = function (observer) {
        observer.unobserve();
    };
    FastJsonPatchService.prototype.getMirror = function (obj) {
        for (var i = 0, length_1 = this.beforeObjects.length; i < length_1; i++) {
            if (this.beforeObjects[i].obj === obj) {
                return this.beforeObjects[i];
            }
        }
    };
    FastJsonPatchService.prototype.getObserverFromMirror = function (mirror, callback) {
        for (var j = 0, length_2 = mirror.observers.length; j < length_2; j++) {
            if (mirror.observers[j].callback === callback) {
                return mirror.observers[j].observer;
            }
        }
    };
    FastJsonPatchService.prototype.removeObserverFromMirror = function (mirror, observer) {
        var observerIndex = mirror.observers.indexOf(observer);
        mirror.observers.splice(observerIndex, 1);
    };
    FastJsonPatchService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    FastJsonPatchService.ctorParameters = function () { return []; };
    return FastJsonPatchService;
}());
export { FastJsonPatchService };
//# sourceMappingURL=angular-fast-json-patch.service.js.map